=== modified file 'lib/lp/soyuz/browser/queue.py'
--- lib/lp/soyuz/browser/queue.py	2010-03-09 08:33:19 +0000
+++ lib/lp/soyuz/browser/queue.py	2010-03-17 02:25:13 +0000
@@ -28,6 +28,7 @@
     IBinaryPackageNameSet)
 from lp.soyuz.interfaces.files import (
     IBinaryPackageFileSet, ISourcePackageReleaseFileSet)
+from lp.soyuz.interfaces.packageset import IPackagesetSet
 from lp.soyuz.interfaces.publishing import name_priority_map
 from canonical.launchpad.webapp import LaunchpadView
 from canonical.launchpad.webapp.batching import BatchNavigator
@@ -480,3 +481,13 @@
             return self.sources[0].sourcepackagerelease.upload_changesfile
         return self.context.changesfile
 
+    @property
+    def package_sets(self):
+        assert self.sourcepackagerelease, \
+            "Can only be used on a source upload."
+        return ' '.join(sorted(ps.name for ps in
+            getUtility(IPackagesetSet).setsIncludingSource(
+                self.sourcepackagerelease.sourcepackagename,
+                distroseries=self.distroseries,
+                direct_inclusion=True)))
+

=== modified file 'lib/lp/soyuz/doc/packageset.txt'
--- lib/lp/soyuz/doc/packageset.txt	2009-12-24 01:41:54 +0000
+++ lib/lp/soyuz/doc/packageset.txt	2010-03-16 01:30:39 +0000
@@ -500,6 +500,20 @@
       1 -> umbrella
       5 -> firefox
 
+We can filter the package sets by series:
+
+    >>> from canonical.launchpad.interfaces import IDistributionSet
+    >>> ubuntu = getUtility(IDistributionSet)['ubuntu']
+    >>> print_data(
+    ...     ps_set.setsIncludingSource(firefox_spn,
+    ...                                distroseries=ubuntu['hoary']))
+      1 -> umbrella
+      4 -> mozilla
+      5 -> firefox
+    >>> print_data(
+    ...     ps_set.setsIncludingSource(firefox_spn,
+    ...                                distroseries=ubuntu['warty']))
+   
 It is also possible to ask the same question by providing the mere name of
 the source package.
 
@@ -622,10 +636,8 @@
 Introduce a copy archive that will be used to disambiguate archive
 permissions.
 
-    >>> from canonical.launchpad.interfaces import IDistributionSet
     >>> from lp.soyuz.interfaces.archive import (
     ...     ArchivePurpose, IArchiveSet)
-    >>> ubuntu = getUtility(IDistributionSet)['ubuntu']
     >>> rebuild_archive = getUtility(IArchiveSet).new(
     ...     owner=person1, purpose=ArchivePurpose.COPY,
     ...     distribution=ubuntu, name='copy-archive',

=== modified file 'lib/lp/soyuz/interfaces/packageset.py'
--- lib/lp/soyuz/interfaces/packageset.py	2009-11-03 14:43:22 +0000
+++ lib/lp/soyuz/interfaces/packageset.py	2010-03-16 01:30:39 +0000
@@ -27,6 +27,7 @@
     operation_parameters, operation_returns_collection_of,
     operation_returns_entry, webservice_error)
 from lazr.restful.fields import Reference
+from lazr.restful.interface import copy_field
 from lp.registry.interfaces.distroseries import IDistroSeries
 from lp.registry.interfaces.person import IPerson
 from lp.registry.interfaces.role import IHasOwner
@@ -413,10 +414,12 @@
     @operation_parameters(
         sourcepackagename=TextLine(
             title=_('Source package name'), required=True),
+        distroseries=copy_field(IPackageset['distroseries'], required=False),
         direct_inclusion=Bool(required=False))
     @operation_returns_collection_of(IPackageset)
     @export_read_operation()
-    def setsIncludingSource(sourcepackagename, direct_inclusion=False):
+    def setsIncludingSource(sourcepackagename, distroseries=None,
+                            direct_inclusion=False):
         """Get the package sets that include this source package.
 
         Return all package sets that directly or indirectly include the
@@ -424,6 +427,9 @@
 
         :param sourcepackagename: the included source package name; can be
             either a string or a `ISourcePackageName`.
+        :param distroseries: the `IDistroSeries` in which to look for sets.
+            If omitted, matching package sets from all series will be
+            returned.
         :param direct_inclusion: if this flag is set to True, then only
             package sets that directly include this source package name will
             be considered.

=== modified file 'lib/lp/soyuz/model/packageset.py'
--- lib/lp/soyuz/model/packageset.py	2009-11-06 21:10:13 +0000
+++ lib/lp/soyuz/model/packageset.py	2010-03-16 01:30:39 +0000
@@ -403,7 +403,8 @@
             source_name = getUtility(ISourcePackageNameSet)[source_name]
         return source_name
 
-    def setsIncludingSource(self, sourcepackagename, direct_inclusion=False):
+    def setsIncludingSource(self, sourcepackagename, distroseries=None,
+                            direct_inclusion=False):
         """See `IPackagesetSet`."""
         sourcepackagename = self._nameToSourcePackageName(sourcepackagename)
 
@@ -421,5 +422,9 @@
             '''
         store = IStore(Packageset)
         psets = SQL(query, (sourcepackagename.id,))
-        result_set = store.find(Packageset, In(Packageset.id, psets))
+        clauses = [In(Packageset.id, psets)]
+        if distroseries:
+            clauses.append(Packageset.distroseries == distroseries)
+
+        result_set = store.find(Packageset, *clauses)
         return _order_result_set(result_set)

=== modified file 'lib/lp/soyuz/stories/soyuz/xx-queue-pages-delayed-copies.txt'
--- lib/lp/soyuz/stories/soyuz/xx-queue-pages-delayed-copies.txt	2010-02-26 13:35:45 +0000
+++ lib/lp/soyuz/stories/soyuz/xx-queue-pages-delayed-copies.txt	2010-03-17 01:53:33 +0000
@@ -48,7 +48,7 @@
 
     >>> for row in find_tags_by_class(anon_browser.contents, "queue-row"):
     ...     print extract_text(row)
-    Package                           Version  Component Section Priority Pocket When
+    Package                           Version  Component Section Priority Sets Pocket When
     foo, foo (delayed) (source, i386) 666      main      base    low      ...
 
     >>> anon_browser.getLink('foo, foo')
@@ -84,7 +84,7 @@
 
     >>> for row in find_tags_by_class(cprov_browser.contents, "queue-row"):
     ...     print extract_text(row)
-    Package                           Version  Component Section Priority Pocket When
+    Package                           Version  Component Section Priority Sets Pocket When
     foo, foo (delayed) (source, i386) 666      main      base    low      ...
 
     >>> anon_browser.getLink('foo, foo')

=== modified file 'lib/lp/soyuz/stories/soyuz/xx-queue-pages-motu.txt'
--- lib/lp/soyuz/stories/soyuz/xx-queue-pages-motu.txt	2009-07-01 13:16:44 +0000
+++ lib/lp/soyuz/stories/soyuz/xx-queue-pages-motu.txt	2010-03-17 01:53:33 +0000
@@ -38,13 +38,13 @@
     ...     "http://launchpad.dev/ubuntu/breezy-autotest/")
     >>> motu_browser.getLink("Show uploads").click()
     >>> print_queue(motu_browser.contents)
-    Package             Version     Component Section Priority Pocket  When
-    netapplet...ddtp... -                                      Release 2006...
-    netapplet...dist... -                                      Release 2006...
-    alsa-utils (source) 1.0.9a-4... main      base    low      Release 2006...
-    netapplet (source)  0.99.6-1    main      web     low      Release 2006...
-    pmount (i386)       0.1-1                                  Release 2006...
-    moz...irefox (i386) 0.9                                    Release 2006...
+    Package             Version     Component Section Priority Sets Pocket  When
+    netapplet...ddtp... -                                           Release 2006...
+    netapplet...dist... -                                           Release 2006...
+    alsa-utils (source) 1.0.9a-4... main      base    low           Release 2006...
+    netapplet (source)  0.99.6-1    main      web     low           Release 2006...
+    pmount (i386)       0.1-1                                       Release 2006...
+    moz...irefox (i386) 0.9                                         Release 2006...
 
 If we try and accept "alsa-utils" it will fail because our user does
 not have permission to accept items in "main":

=== modified file 'lib/lp/soyuz/stories/soyuz/xx-queue-pages.txt'
--- lib/lp/soyuz/stories/soyuz/xx-queue-pages.txt	2010-03-05 17:04:27 +0000
+++ lib/lp/soyuz/stories/soyuz/xx-queue-pages.txt	2010-03-17 01:53:33 +0000
@@ -62,13 +62,13 @@
   ...         print extract_text(row)
 
   >>> print_queue(anon_browser.contents)
-  Package             Version     Component Section Priority Pocket  When
-  netapplet...ddtp... -                                      Release 2006-...
-  netapplet...dist... -                                      Release 2006-...
-  alsa-utils (source) 1.0.9a-4... main      base    low      Release 2006-...
-  netapplet (source)  0.99.6-1    main      web     low      Release 2006-...
-  pmount (i386)       0.1-1                                  Release 2006-...
-  moz...irefox (i386) 0.9                                    Release 2006-...
+  Package             Version     Component Section Priority Sets Pocket  When
+  netapplet...ddtp... -                                           Release 2006-...
+  netapplet...dist... -                                           Release 2006-...
+  alsa-utils (source) 1.0.9a-4... main      base    low           Release 2006-...
+  netapplet (source)  0.99.6-1    main      web     low           Release 2006-...
+  pmount (i386)       0.1-1                                       Release 2006-...
+  moz...irefox (i386) 0.9                                         Release 2006-...
 
 The package name in the results list is a clickable link to the changes
 file for that upload.
@@ -92,12 +92,12 @@
   ...     name="queue_state", index=0).displayValue = ['Unapproved']
   >>> anon_browser.getControl("Update").click()
   >>> print_queue(anon_browser.contents)
-  Package             Version   Component Section  Priority Pocket   When
-  lang...-de (source) 1.0       main      trans... low      Proposed 2007-...
-  netapplet...ddtp... -                                     Backp... 2006-...
-  cnews (source)      1.0       main      base     low      Release  2006-...
-  cnews (source)      1.0       main      base     low      Release  2006-...
-  netapplet...(raw-translations) -                          Updates  2006-...
+  Package             Version   Component Section  Priority Sets Pocket   When
+  lang...-de (source) 1.0       main      trans... low           Proposed 2007-...
+  netapplet...ddtp... -                                          Backp... 2006-...
+  cnews (source)      1.0       main      base     low           Release  2006-...
+  cnews (source)      1.0       main      base     low           Release  2006-...
+  netapplet...(raw-translations) -                               Updates  2006-...
 
 The results can be filtered matching source name, binary name or
 custom-upload filename.
@@ -105,24 +105,68 @@
   >>> anon_browser.getControl(name="queue_text").value = 'language'
   >>> anon_browser.getControl("Update").click()
   >>> print_queue(anon_browser.contents)
-  Package             Version   Component Section  Priority Pocket   When
-  lang...-de (source) 1.0       main      trans... low      Proposed 2007-...
+  Package             Version   Component Section  Priority Sets Pocket   When
+  lang...-de (source) 1.0       main      trans... low           Proposed 2007-...
 
   >>> anon_browser.getControl(name="queue_text").value = 'netapplet'
   >>> anon_browser.getControl("Update").click()
   >>> print_queue(anon_browser.contents)
-  Package             Version   Component Section  Priority Pocket   When
-  netapplet...(raw-translations) -                          Updates  2006-...
-  netapplet...ddtp... -                                     Backp... 2006-...
+  Package             Version   Component Section  Priority Sets Pocket   When
+  netapplet...(raw-translations) -                               Updates  2006-...
+  netapplet...ddtp... -                                          Backp... 2006-...
 
   >>> anon_browser.getControl(
   ...     name="queue_state", index=0).displayValue = ['New']
   >>> anon_browser.getControl(name="queue_text").value = 'pmount'
   >>> anon_browser.getControl("Update").click()
   >>> print_queue(anon_browser.contents)
-  Package             Version   Component Section  Priority Pocket   When
-  pmount (i386)       0.1-1                                 Release  2006-...
-
+  Package             Version   Component Section  Priority Sets Pocket   When
+  pmount (i386)       0.1-1                                      Release  2006-...
+
+A source's package sets are listed in the queue. Since there are none in
+the sample data, we'll first add some.
+
+  >>> login('foo.bar@canonical.com')
+  >>> from lp.soyuz.interfaces.packageset import IPackagesetSet
+  >>> ubuntu = getUtility(IDistributionSet)['ubuntu']
+  >>> hoary = ubuntu['hoary']
+  >>> breezy_autotest = ubuntu['breezy-autotest']
+  >>> pss = getUtility(IPackagesetSet)
+  >>> desktop = pss.new(
+  ...     u'desktop', u'Ubuntu Desktop', name12, breezy_autotest)
+  >>> server = pss.new(
+  ...     u'server', u'Ubuntu Server', name12, breezy_autotest)
+  >>> core = pss.new(
+  ...     u'core', u'Ubuntu Core', name12, breezy_autotest)
+  >>> desktop.add([core])
+  >>> desktop.addSources(['alsa-utils'])
+  >>> server.addSources(['alsa-utils'])
+  >>> core.addSources(['netapplet'])
+
+Package sets from other series are not shown.
+
+  >>> kubuntu = pss.new(u'kubuntu', u'Kubuntu', name12, hoary)
+  >>> kubuntu.addSources(['alsa-utils'])
+  >>> logout()
+
+  >>> anon_browser.getControl(
+  ...     name="queue_state", index=0).displayValue = ['New']
+  >>> anon_browser.getControl(name="queue_text").value = ''
+  >>> anon_browser.getControl("Update").click()
+  >>> print_queue(anon_browser.contents)
+  Package             Version   Component Section  Priority Sets           Pocket  When
+  netapplet...ddtp... -                                                    Release 2006-...
+  netapplet...dist... -                                                    Release 2006-...
+  alsa-utils (source) 1.0.9a-4... main      base    low     desktop server Release 2006-...
+  netapplet (source)  0.99.6-1    main      web     low     core           Release 2006-...
+  pmount (i386)       0.1-1                                                Release 2006-...
+  moz...irefox (i386) 0.9                                                  Release 2006-...
+
+  >>> login('foo.bar@canonical.com')
+  >>> desktop.removeSources(['alsa-utils'])
+  >>> server.removeSources(['alsa-utils'])
+  >>> core.removeSources(['netapplet'])
+  >>> logout()
 
 == Queue item filelist ==
 
@@ -243,26 +287,26 @@
   >>> upload_manager_browser.open(
   ...    "http://localhost/ubuntu/breezy-autotest/+queue")
   >>> print_queue(upload_manager_browser.contents)
-  Package             Version     Component Section Priority Pocket  When
-  bar (source)        1.0-1       universe  devel   low      Release ...
-  netapplet...ddtp... -                                      Release 2006-...
-  netapplet...dist... -                                      Release 2006-...
-  alsa-utils (source) 1.0.9a-4... main      base    low      Release 2006-...
-  netapplet (source)  0.99.6-1    main      web     low      Release 2006-...
-  pmount (i386)       0.1-1                                  Release 2006-...
-  moz...irefox (i386) 0.9                                    Release 2006-...
+  Package             Version     Component Section Priority Sets Pocket  When
+  bar (source)        1.0-1       universe  devel   low           Release ...
+  netapplet...ddtp... -                                           Release 2006-...
+  netapplet...dist... -                                           Release 2006-...
+  alsa-utils (source) 1.0.9a-4... main      base    low           Release 2006-...
+  netapplet (source)  0.99.6-1    main      web     low           Release 2006-...
+  pmount (i386)       0.1-1                                       Release 2006-...
+  moz...irefox (i386) 0.9                                         Release 2006-...
 
   >>> upload_manager_browser.getControl(
   ...     name="QUEUE_ID").value = [str(bar_queue_id)]
   >>> upload_manager_browser.getControl(name="Accept").click()
   >>> print_queue(upload_manager_browser.contents)
-  Package             Version     Component Section Priority Pocket  When
-  netapplet...ddtp... -                                      Release 2006-...
-  netapplet...dist... -                                      Release 2006-...
-  alsa-utils (source) 1.0.9a-4... main      base    low      Release 2006-...
-  netapplet (source)  0.99.6-1    main      web     low      Release 2006-...
-  pmount (i386)       0.1-1                                  Release 2006-...
-  moz...irefox (i386) 0.9                                    Release 2006-...
+  Package             Version     Component Section Priority Sets Pocket  When
+  netapplet...ddtp... -                                           Release 2006-...
+  netapplet...dist... -                                           Release 2006-...
+  alsa-utils (source) 1.0.9a-4... main      base    low           Release 2006-...
+  netapplet (source)  0.99.6-1    main      web     low           Release 2006-...
+  pmount (i386)       0.1-1                                       Release 2006-...
+  moz...irefox (i386) 0.9                                         Release 2006-...
 
 Accepting queue items results in an email to the uploader (and the changer
 if it is someone other than the uploader) and (usually) an email to the
@@ -310,8 +354,8 @@
   ...    name="queue_state", index=0).displayValue = ['Accepted']
   >>> upload_manager_browser.getControl("Update").click()
   >>> print_queue(upload_manager_browser.contents)
-  Package             Version     Component Section Priority Pocket  When
-  moz...irefox (i386) 0.9                                    Release 2006-...
+  Package             Version     Component Section Priority Sets Pocket  When
+  moz...irefox (i386) 0.9                                         Release 2006-...
 
 Going back to the "new" queue, we can see our item has gone:
 
@@ -319,8 +363,8 @@
   ...    name="queue_state", index=0).displayValue = ['New']
   >>> upload_manager_browser.getControl("Update").click()
   >>> print_queue(upload_manager_browser.contents)
-  Package             Version     Component Section Priority Pocket  When
-  netapplet...ddtp... -                                      Release 2006-...
+  Package             Version     Component Section Priority Sets Pocket  When
+  netapplet...ddtp... -                                           Release 2006-...
   netapplet...dist... -                                      Release 2006-...
   alsa-utils (source) 1.0.9a-4... main      base    low      Release 2006-...
   netapplet (source)  0.99.6-1    main      web     low      Release 2006-...
@@ -396,10 +440,10 @@
   OK: pmount(restricted/admin/extra)
 
   >>> print_queue(upload_manager_browser.contents)
-  Package             Version     Component Section Priority Pocket  When
-  netapplet...ddtp... -                                      Release 2006-...
-  netapplet...dist... -                                      Release 2006-...
-  alsa-utils (source) 1.0.9a-4... main      base    low      Release 2006-...
+  Package             Version     Component Section Priority Sets Pocket  When
+  netapplet...ddtp... -                                           Release 2006-...
+  netapplet...dist... -                                           Release 2006-...
+  alsa-utils (source) 1.0.9a-4... main      base    low           Release 2006-...
 
 Any user can now see the 'accepted' queue contains pmount with its
 overridden values.
@@ -408,8 +452,8 @@
   ...     name="queue_state", index=0).displayValue=['Accepted']
   >>> anon_browser.getControl("Update").click()
   >>> print_queue(anon_browser.contents)
-  Package             Version     Component Section Priority Pocket  When
-  pmount (i386)       0.1-1                                  Release 2006-...
+  Package             Version     Component Section Priority Sets Pocket  When
+  pmount (i386)       0.1-1                                       Release 2006-...
   ...
 
 The user can drill down into the file list to see the overridden binary
@@ -428,7 +472,7 @@
   ...     name="queue_state", index=0).displayValue=['Done']
   >>> anon_browser.getControl("Update").click()
   >>> print_queue(anon_browser.contents)
-  Package             Version     Component  Section Priority Pocket  When
+  Package             Version     Component  Section Priority Sets Pocket  When
   ...
   netapplet (source)  0.99.6-1    restricted admin   low ...
 
@@ -446,9 +490,9 @@
   OK: alsa-utils
 
   >>> print_queue(upload_manager_browser.contents)
-  Package             Version     Component Section Priority Pocket  When
-  netapplet...ddtp... -                                      Release 2006-...
-  netapplet...dist... -                                      Release 2006-...
+  Package             Version     Component Section Priority Sets Pocket  When
+  netapplet...ddtp... -                                           Release 2006-...
+  netapplet...dist... -                                           Release 2006-...
 
 One rejection email is generated:
 

=== modified file 'lib/lp/soyuz/templates/distroseries-queue.pt'
--- lib/lp/soyuz/templates/distroseries-queue.pt	2009-09-04 10:19:08 +0000
+++ lib/lp/soyuz/templates/distroseries-queue.pt	2010-03-16 01:13:05 +0000
@@ -63,6 +63,7 @@
            <th> Component </th>
            <th> Section </th>
            <th> Priority </th>
+           <th> Sets </th>
            <th> Pocket </th>
            <th> When </th>
          </tr>
@@ -111,6 +112,11 @@
                          content="packageupload/sourcepackagerelease/urgency/name/lower">
                     </tal:priority>
                   </td>
+                  <td style="padding-top: 5px">
+                    <tal:packagesets condition="is_source"
+                         content="packageupload/package_sets">
+                    </tal:packagesets>
+                  </td>
                 </tal:is_source>
                 <td style="padding-top: 5px"
                     tal:content="packageupload/pocket/title">Updates

